@@ -122,20 +122,6 @@ class urlopen_HttpTests(unittest.TestCase):
         finally:
             self.unfakehttp()
 
-    def test_invalid_redirect(self):
-        # urlopen() should raise IOError for many error codes.
-        self.fakehttp("""HTTP/1.1 302 Found
-Date: Wed, 02 Jan 2008 03:03:54 GMT
-Server: Apache/1.3.33 (Debian GNU/Linux) mod_ssl/2.8.22 OpenSSL/0.9.7e
-Location: file:README
-Connection: close
-Content-Type: text/html; charset=iso-8859-1
-""")
-        try:
-            self.assertRaises(IOError, urllib.urlopen, "http://python.org/")
-        finally:
-            self.unfakehttp()
-
 class urlretrieve_FileTests(unittest.TestCase):
     """Test urllib.urlretrieve() on local files"""
 
@@ -367,6 +353,12 @@ class QuotingTests(unittest.TestCase):
         self.assertEqual(expect, result,
                          "using quote_plus(): %s != %s" % (expect, result))
 
+    def test_quoting_plus(self):
+        self.assertEqual(urllib.quote_plus('alpha+beta gamma'),
+                         'alpha%2Bbeta+gamma')
+        self.assertEqual(urllib.quote_plus('alpha+beta gamma', '+'),
+                         'alpha+beta+gamma')
+
 class UnquotingTests(unittest.TestCase):
     """Tests for unquote() and unquote_plus()
 
